aboutsummaryrefslogtreecommitdiff
path: root/sysdeps
diff options
context:
space:
mode:
authorAdhemerval Zanella <adhemerval.zanella@linaro.org>2019-11-05 21:37:44 +0000
committerAdhemerval Zanella <adhemerval.zanella@linaro.org>2019-11-06 14:47:02 -0300
commit3537ecb49cf7177274607004c562d6f9ecc99474 (patch)
tree7321853007814993f485345e97b9b2ff61032a0c /sysdeps
parent79a547b162657b3fa34d31917cc29f0e7af19e4c (diff)
downloadglibc-3537ecb49cf7177274607004c562d6f9ecc99474.zip
glibc-3537ecb49cf7177274607004c562d6f9ecc99474.tar.gz
glibc-3537ecb49cf7177274607004c562d6f9ecc99474.tar.bz2
Refactor nanosleep in terms of clock_nanosleep
The generic version is straightforward. For Hurd, its nanosleep implementation is moved to clock_nanosleep with adjustments from generic unix implementation. The generic clock_nanosleep unix version is also removed since it calls nanosleep. Checked on x86_64-linux-gnu and powerpc64le-linux-gnu. Reviewed-by: Florian Weimer <fweimer@redhat.com>
Diffstat (limited to 'sysdeps')
-rw-r--r--sysdeps/mach/clock_nanosleep.c (renamed from sysdeps/unix/clock_nanosleep.c)67
-rw-r--r--sysdeps/mach/nanosleep.c79
-rw-r--r--sysdeps/unix/sysv/linux/clock_nanosleep.c2
-rw-r--r--sysdeps/unix/sysv/linux/nanosleep.c31
4 files changed, 47 insertions, 132 deletions
diff --git a/sysdeps/mach/nanosleep.c b/sysdeps/mach/nanosleep.c
deleted file mode 100644
index b60a217..0000000
--- a/sysdeps/mach/nanosleep.c
+++ /dev/null
@@ -1,79 +0,0 @@
-/* nanosleep -- sleep for a period specified with a struct timespec
- Copyright (C) 2002-2019 Free Software Foundation, Inc.
- This file is part of the GNU C Library.
-
- The GNU C Library is free software; you can redistribute it and/or
- modify it under the terms of the GNU Lesser General Public
- License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
-
- The GNU C Library is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- Lesser General Public License for more details.
-
- You should have received a copy of the GNU Lesser General Public
- License along with the GNU C Library; if not, see
- <https://www.gnu.org/licenses/>. */
-
-#include <errno.h>
-#include <mach.h>
-#include <time.h>
-#include <unistd.h>
-
-# define timespec_sub(a, b, result) \
- do { \
- (result)->tv_sec = (a)->tv_sec - (b)->tv_sec; \
- (result)->tv_nsec = (a)->tv_nsec - (b)->tv_nsec; \
- if ((result)->tv_nsec < 0) { \
- --(result)->tv_sec; \
- (result)->tv_nsec += 1000000000; \
- } \
- } while (0)
-
-int
-__libc_nanosleep (const struct timespec *requested_time,
- struct timespec *remaining)
-{
- mach_port_t recv;
- struct timespec before;
- error_t err;
-
- if (requested_time->tv_sec < 0
- || ! valid_nanoseconds (requested_time->tv_nsec))
- {
- errno = EINVAL;
- return -1;
- }
-
- const mach_msg_timeout_t ms
- = requested_time->tv_sec * 1000
- + (requested_time->tv_nsec + 999999) / 1000000;
-
- recv = __mach_reply_port ();
-
- if (remaining != 0)
- __clock_gettime (CLOCK_REALTIME, &before);
-
- err = __mach_msg (NULL, MACH_RCV_MSG|MACH_RCV_TIMEOUT|MACH_RCV_INTERRUPT,
- 0, 0, recv, ms, MACH_PORT_NULL);
- __mach_port_destroy (mach_task_self (), recv);
- if (err == EMACH_RCV_INTERRUPTED)
- {
- if (remaining != 0)
- {
- struct timespec after, elapsed;
- __clock_gettime (CLOCK_REALTIME, &after);
- timespec_sub (&after, &before, &elapsed);
- timespec_sub (requested_time, &elapsed, remaining);
- }
-
- errno = EINTR;
- return -1;
- }
-
- return 0;
-}
-weak_alias(__libc_nanosleep, __nanosleep)
-libc_hidden_def (__nanosleep)
-weak_alias (__libc_nanosleep, nanosleep)
diff --git a/sysdeps/unix/sysv/linux/clock_nanosleep.c b/sysdeps/unix/sysv/linux/clock_nanosleep.c
index 1f240b8..f3c6fd2 100644
--- a/sysdeps/unix/sysv/linux/clock_nanosleep.c
+++ b/sysdeps/unix/sysv/linux/clock_nanosleep.c
@@ -42,7 +42,7 @@ __clock_nanosleep (clockid_t clock_id, int flags, const struct timespec *req,
return (INTERNAL_SYSCALL_ERROR_P (r, err)
? INTERNAL_SYSCALL_ERRNO (r, err) : 0);
}
-
+libc_hidden_def (__clock_nanosleep)
versioned_symbol (libc, __clock_nanosleep, clock_nanosleep, GLIBC_2_17);
/* clock_nanosleep moved to libc in version 2.17;
old binaries may expect the symbol version it had in librt. */
diff --git a/sysdeps/unix/sysv/linux/nanosleep.c b/sysdeps/unix/sysv/linux/nanosleep.c
deleted file mode 100644
index 6787909..0000000
--- a/sysdeps/unix/sysv/linux/nanosleep.c
+++ /dev/null
@@ -1,31 +0,0 @@
-/* Linux nanosleep syscall implementation.
- Copyright (C) 2017-2019 Free Software Foundation, Inc.
- This file is part of the GNU C Library.
-
- The GNU C Library is free software; you can redistribute it and/or
- modify it under the terms of the GNU Lesser General Public
- License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
-
- The GNU C Library is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- Lesser General Public License for more details.
-
- You should have received a copy of the GNU Lesser General Public
- License along with the GNU C Library; if not, see
- <https://www.gnu.org/licenses/>. */
-
-#include <time.h>
-#include <sysdep-cancel.h>
-#include <not-cancel.h>
-
-/* Pause execution for a number of nanoseconds. */
-int
-__nanosleep (const struct timespec *requested_time,
- struct timespec *remaining)
-{
- return SYSCALL_CANCEL (nanosleep, requested_time, remaining);
-}
-hidden_def (__nanosleep)
-weak_alias (__nanosleep, nanosleep)
diff --git a/sysdeps/unix/clock_nanosleep.c b/sysdeps/mach/clock_nanosleep.c
index 8514a43..6dc67d6 100644
--- a/sysdeps/unix/clock_nanosleep.c
+++ b/sysdeps/mach/clock_nanosleep.c
@@ -1,5 +1,5 @@
-/* High-resolution sleep with the specified clock.
- Copyright (C) 2000-2019 Free Software Foundation, Inc.
+/* clock_nanosleep - high-resolution sleep with specifiable clock.
+ Copyright (C) 2002-2019 Free Software Foundation, Inc.
This file is part of the GNU C Library.
The GNU C Library is free software; you can redistribute it and/or
@@ -16,28 +16,58 @@
License along with the GNU C Library; if not, see
<https://www.gnu.org/licenses/>. */
-#include <assert.h>
#include <errno.h>
+#include <mach.h>
#include <time.h>
-#include <sysdep-cancel.h>
+#include <unistd.h>
+#include <posix-timer.h>
#include <shlib-compat.h>
-/* This implementation assumes that these is only a `nanosleep' system
- call. So we have to remap all other activities. */
+static int
+nanosleep_call (const struct timespec *req, struct timespec *rem)
+{
+ mach_port_t recv;
+ struct timespec before;
+ error_t err;
+
+ const mach_msg_timeout_t ms
+ = req->tv_sec * 1000
+ + (req->tv_nsec + 999999) / 1000000;
+
+ recv = __mach_reply_port ();
+
+ if (rem != NULL)
+ __clock_gettime (CLOCK_REALTIME, &before);
+
+ err = __mach_msg (NULL, MACH_RCV_MSG|MACH_RCV_TIMEOUT|MACH_RCV_INTERRUPT,
+ 0, 0, recv, ms, MACH_PORT_NULL);
+ __mach_port_destroy (mach_task_self (), recv);
+ if (err == EMACH_RCV_INTERRUPTED)
+ {
+ if (rem != NULL)
+ {
+ struct timespec after, elapsed;
+ __clock_gettime (CLOCK_REALTIME, &after);
+ timespec_sub (&elapsed, &after, &before);
+ timespec_sub (rem, req, &elapsed);
+ }
+
+ return EINTR;
+ }
+
+ return 0;
+}
+
int
__clock_nanosleep (clockid_t clock_id, int flags, const struct timespec *req,
struct timespec *rem)
{
- struct timespec now;
-
- if (! valid_nanoseconds (req->tv_nsec))
+ if (clock_id != CLOCK_REALTIME
+ || !valid_nanoseconds (req->tv_nsec)
+ || (flags != 0 && flags != TIMER_ABSTIME))
return EINVAL;
- if (clock_id == CLOCK_THREAD_CPUTIME_ID)
- return EINVAL; /* POSIX specifies EINVAL for this case. */
-
- if (clock_id < CLOCK_REALTIME || clock_id > CLOCK_THREAD_CPUTIME_ID)
- return EINVAL;
+ struct timespec now;
/* If we got an absolute time, remap it. */
if (flags == TIMER_ABSTIME)
@@ -68,15 +98,10 @@ __clock_nanosleep (clockid_t clock_id, int flags, const struct timespec *req,
/* Make sure we are not modifying the struct pointed to by REM. */
rem = NULL;
}
- else if (flags != 0)
- return EINVAL;
- else if (clock_id != CLOCK_REALTIME)
- /* Not supported. */
- return ENOTSUP;
- return __nanosleep (req, rem), 0 ? errno : 0;
+ return nanosleep_call (req, rem);
}
-
+libc_hidden_def (__clock_nanosleep)
versioned_symbol (libc, __clock_nanosleep, clock_nanosleep, GLIBC_2_17);
/* clock_nanosleep moved to libc in version 2.17;
old binaries may expect the symbol version it had in librt. */